<?php

namespace Drupal\og_nonmember_editor\Plugin\Validation\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Checks if referenced entities are valid.
 */
class GroupContentModerationConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($entity, Constraint $constraint) {
    $groups = $entity->referencedEntities();
    $group = reset($groups);

    // Do nothing if no group is chosen.
    if (empty($groups)) {
      return;
    }
    // Get the base entity.
    $root_entity = $entity->getEntity();

    // // get the current user.
    $user = \Drupal::currentUser()->getAccount();

    // @var \Drupal\og_nonmember_editor\NonMemberStateTransitionValidation $transition_validation.
    $transition_validation = \Drupal::service('og_nonmember_editor.state_transition_validation');

    // Get the transitions a user is allowed to do.
    $transitions = $transition_validation->getValidTransitions($root_entity, $user);

    // Iterate through each transition permission and validate user action.
    $is_allowed = TRUE;
    foreach ($transitions as $transition) {
      if ($root_entity->moderation_state->value != $transition->to()->id()) {
        $is_allowed = FALSE;
      }
    }

    // Throw an error when a user does not have permission to access content.
    if (!$is_allowed) {
      $this->context->addViolation($constraint->NoPermissionToCreateTransition, ["%group" => $group->label()]);
    }
  }

}
